@@ -6,7 +6,7 @@
 
 /* factoring_sub.c --- functions to build successor tree and compute factoring value */
 
-/* $Id: factoring_sub.c,v 1.10 2003/09/29 06:01:22 ri Exp $ */
+/* $Id: factoring_sub.c,v 1.11 2003/12/05 01:40:23 ri Exp $ */
 
 /* for Julius (N-gram) : 2-gram factoring, 1-gram factoring implemented
    for Julian (grammar): deterministic factoring implemented
@@ -532,7 +532,7 @@ calc_successor_prob(WCHMM_INFO *wchmm, WORD_ID last_nword, int node)
   for (sc = wchmm->sclist[wchmm->state[node].scid]; sc; sc = sc->next) {
     tmpprob = bi_prob_lr(wchmm->ngram, last_nword, wchmm->winfo->wton[sc->word])
 #ifdef CLASS_NGRAM
-      + wchmm->winfo->cprob[sctmp->word]
+      + wchmm->winfo->cprob[sc->word]
 #endif
       ;
     if (maxprob < tmpprob) maxprob = tmpprob;
