@@ -6,7 +6,7 @@
 
 /* global.h --- global variables */
 
-/* $Id: global.h,v 1.21 2003/10/04 12:09:33 ri Exp $ */
+/* $Id: global.h,v 1.26 2004/01/08 08:42:17 ri Exp $ */
 
 #ifndef __SENT_EXTERNAL_DEFINITION__
 #define __SENT_EXTERNAL_DEFINITION__
@@ -76,6 +76,7 @@ GLOBAL int head_margin_msec GLOBAL_VAL(300); /* head margin in msec */
 GLOBAL int tail_margin_msec GLOBAL_VAL(400); /* tail margin in msec */
 
 GLOBAL boolean strip_zero_sample GLOBAL_VAL(TRUE); /* strip off zero samples */
+GLOBAL boolean use_zmean GLOBAL_VAL(FALSE); /* remove DC by zero mean */
 
 /* speech analysis parameters */
 GLOBAL int smpPeriod GLOBAL_VAL(625); /* sampling period (16kHz: 625ns) */
@@ -197,10 +198,11 @@ GLOBAL boolean old_iwcd_flag GLOBAL_VAL(FALSE); /* use full lcdset instead of *c
 #endif
 #endif
 #ifdef USE_NGRAM
-GLOBAL boolean iwcdavg_flag GLOBAL_VAL(FALSE); /* use average score of lcdset in cross-word triphone: default is TRUE for DFA, FALSE for N-gram */
+GLOBAL short iwcdmethod GLOBAL_VAL(IWCD_NBEST); /* use average score of lcdset in cross-word triphone */
 #else
-GLOBAL boolean iwcdavg_flag GLOBAL_VAL(TRUE);
+GLOBAL short iwcdmethod GLOBAL_VAL(IWCD_AVG);
 #endif
+GLOBAL short iwcdmaxn GLOBAL_VAL(3);
 #ifdef USE_DFA
 GLOBAL boolean looktrellis_flag GLOBAL_VAL(FALSE); /* restrict expansion on only neighbor frames (-looktrellis) for 2nd pass */
 #endif
@@ -267,9 +269,10 @@ GLOBAL LOGPROB *framemaxscore;/* maximum score of each frame on 2nd pass */
 
 /* input speech related */
 GLOBAL SP16 speech[MAXSPEECHLEN]; /* speech data */
-GLOBAL int speechlen;		/* speech length */
+GLOBAL int speechlen GLOBAL_VAL(0);		/* speech length */
 GLOBAL boolean cmn_loaded GLOBAL_VAL(FALSE); /* TRUE if CMN loaded from file */
 GLOBAL boolean c0_required GLOBAL_VAL(FALSE); /* TRUE when 0'th Cepstral parameter is used instead of energy in hmmdefs */
+GLOBAL boolean cmn_required GLOBAL_VAL(TRUE);
 
 /* spectral subtraction */
 GLOBAL float *ssbuf GLOBAL_VAL(NULL);/* noise spectrum */
@@ -290,7 +293,11 @@ GLOBAL WORD_ID pass1_wseq[MAXSEQNUM]; /* word seq. of best hypo on 1st pass */
 GLOBAL int pass1_wnum; /* word num of above */
 GLOBAL LOGPROB pass1_score;	/* score of above */
 
-/* pointer to output functions (switche by 'result_output') */
+/* pointer to output functions (switched by 'result_output') */
+GLOBAL void (*status_recready)();
+GLOBAL void (*status_recstart)();
+GLOBAL void (*status_recend)();
+GLOBAL void (*status_param)(HTK_Param *p);
 GLOBAL void (*result_pass1_begin)();
 GLOBAL void (*result_pass1_current)(int t, WORD_ID *seq, int num, LOGPROB score, LOGPROB LMscore, WORD_INFO *winfo);
 GLOBAL void (*result_pass1_final)(WORD_ID *seq, int num, LOGPROB score, LOGPROB LMscore, WORD_INFO *winfo);
