@@ -7,7 +7,7 @@
 
 /* main.c --- main routine */
 
-/* $Id: main.c,v 1.12 2003/09/29 06:01:22 ri Exp $ */
+/* $Id: main.c,v 1.17 2004/02/25 06:02:45 ri Exp $ */
 
 #define GLOBAL_VARIABLE_DEFINE	/* actually make global vars in global.h */
 #include <julius.h>
@@ -68,7 +68,7 @@ static int overflowed_samplenum;
    0  ... continue reading
    1  ... segment detected -> force adin_go to segment (do not flush the cycle buffer)
 */
-static int
+int
 adin_cut_callback_store_buffer(SP16 *now, int len)
 {
   if (module_mode) {
@@ -76,9 +76,10 @@ adin_cut_callback_store_buffer(SP16 *now, int len)
     msock_check_and_process_command();
   }
   if (speechlen == 0) {		/* first part of a segment */
+    /* output start recording/processing message */
+    status_recstart();
     if (module_mode) {
-      /* tell module to start recording */
-      msock_status_recstart();
+      /* module termination check */
       if (module_wants_terminate() ||/* TERMINATE ... force termination */
 	  !module_is_active()) { /* PAUSE ... keep recording when *triggering */
 	return(-2);
@@ -185,6 +186,11 @@ main_recognition_loop()
 
   /* reset file count */
   file_counter = 0;
+
+#ifdef VISUALIZE
+  /* Visualize: initialize GTK */
+  visual_init();
+#endif
   
   /***************************/
   /***************************/
@@ -233,6 +239,10 @@ main_recognition_loop()
 	param_strip_zero(param);
       }
       free(speechfilename);
+      /* output frame length */
+      status_param(param);
+      /* count number of processed files */
+      file_counter++;
     } else {			/* raw speech input */
       VERMES("### read waveform input\n");
       /* begin A/D input */
@@ -247,6 +257,10 @@ main_recognition_loop()
 	  j_error("failed to begin input stream\n");
 	}
       }
+      /* count number of processed files */
+      if (speech_input == SP_RAWFILE) {
+	file_counter++;
+      }
     }
     
     /*********************************/
@@ -296,10 +310,6 @@ main_recognition_loop()
 	/***********************/
 	/* raw wave data input */
 	/***********************/
-	/* if needed, begin recording the incoming speech data to a file */
-	if (record_dirname != NULL) {
-	  record_sample_open();
-	}
 	if (realtime_flag) {
 	  /****************************/
 	  /* SPECIAL: realtime 1st-pass */
@@ -315,9 +325,10 @@ main_recognition_loop()
 	      j_error("error in resuming last fragment\n"); /* exit now! */
 	    }
 	    if (ret != 1) {	/* if segmented again in the margin, not process the rest */
+	      /* output listening start message */
+	      status_recready();
 	      if (module_mode) {
 		/* tell module to start recording, and command check in adin */
-		msock_status_recready();
 		ret = adin_go(RealTimePipeLine, msock_check_in_adin);
 	      } else {
 		/* process the incoming input as normal */
@@ -335,12 +346,13 @@ main_recognition_loop()
 	    /* last was not segmented, process the incoming input  */
 #endif
 	    RealTimePipeLinePrepare();
+	    /* output listening start message */
+	    status_recready();
+	    /* process the incoming input */
 	    if (module_mode) {
-	      /* tell module to start recording, and command check in adin */
-	      msock_status_recready();
 	      ret = adin_go(RealTimePipeLine, msock_check_in_adin);
 	    } else {
-	      ret = adin_go(RealTimePipeLine, NULL); /* process the incoming input */
+	      ret = adin_go(RealTimePipeLine, NULL); 
 	    }
 	    if (ret < 0) {		/* error end in adin_go */
 	      if (module_mode && ret == -2) {	/* terminated by module */
@@ -353,11 +365,13 @@ main_recognition_loop()
 #endif
 	  /* last procedure of 1st-pass */
 	  param = RealTimeParam(&backmax);
+	  /* output stopped recording message */
+	  status_recend();
+	  /* output frame length */
+	  status_param(param);
 	  if (module_mode) {
 	    /* if terminate signal has been received, discard this input */
 	    if (module_wants_terminate()) goto end_recog;
-	    /* tell module that recording is stopped */
-	    msock_status_recend();
 	  }
 	  /* end of 1st pass, jump to 2nd pass */
 	  goto end_1pass;
@@ -371,9 +385,14 @@ main_recognition_loop()
 	if (rest_param == NULL) { /* no segment left */
 #endif
 	  speechlen = 0;
+	  /* if needed, begin recording the incoming speech data to a file */
+	  if (record_dirname != NULL) {
+	    record_sample_open();
+	  }
+	  /* output listening start message */
+	  status_recready();
 	  if (module_mode) {
 	    /* tell module to start recording */
-	    msock_status_recready();
 	    ret = adin_go(adin_cut_callback_store_buffer, msock_check_in_adin);
 	  } else {
 	    ret = adin_go(adin_cut_callback_store_buffer, NULL);
@@ -384,8 +403,8 @@ main_recognition_loop()
 	    }
 	    j_error("error in adin_go\n");              /* exit now! */
 	  }
-	  /* tell module that recording is stopped */
-	  if (module_mode) msock_status_recend();
+	  /* output stopped recording message */
+	  status_recend();
       
 	  j_printf("%d samples (%.2f sec.)\n", speechlen, (float)speechlen / (float) smpFreq);
 	  if (speechlen == 0) j_exit();	/* terminate when no input */
@@ -400,6 +419,8 @@ main_recognition_loop()
 	  param = new_wav2mfcc(speech, speechlen);
 	  if (param == NULL) continue;
 
+	  /* output frame length */
+	  status_param(param);
 	  /* if terminate signal has been received, cancel this input */
 	  if (module_mode && module_wants_terminate()) goto end_recog;
 
@@ -411,9 +432,6 @@ main_recognition_loop()
       /********************************/
       /* check the analized parameter */
       /********************************/
-      if (verbose_flag) {
-	put_param_info(param);
-      }
       /* parameter type check --- compare the type to that of HMM,
 	 and adjust them if necessary */
       if (paramtype_check_flag) {
@@ -464,7 +482,6 @@ main_recognition_loop()
 
     end_1pass:
       /* with realtime_flag, it joins at this point */
-
       /* if [-1pass] is specified, terminate search here */
       if (compute_only_1pass) {
 	goto end_recog;
@@ -489,6 +506,11 @@ main_recognition_loop()
       envl_init(backtrellis.framelen);
 #endif /* SCAN_BEAM */
     
+#ifndef PASS2_STRICT_IWCD
+      /* adjust trellis score not to contain outprob of the last frames */
+      bt_discount_pescore(wchmm, &backtrellis, param);
+#endif
+
 #ifdef USE_NGRAM
       VERMES("### Recognition: 2nd pass (RL heuristic best-first with 3-gram)\n");
 #else
@@ -497,7 +519,6 @@ main_recognition_loop()
 
       /* execute stack-decoding search */
       wchmm_fbs(param, &backtrellis, backmax, stack_size, nbest, hypo_overflow);
-
 #ifdef SCAN_BEAM
       free(framemaxscore);
 #endif
@@ -506,6 +527,11 @@ main_recognition_loop()
       /**********************/
       /* end of recognition */
       /**********************/
+
+#ifdef VISUALIZE
+      visual_show(&backtrellis);
+#endif
+	
       free_param(param);
 
       /* if needed, close the recording files */
@@ -513,11 +539,6 @@ main_recognition_loop()
 	record_sample_close();
       }
 
-      /* count number of processed files */
-      if (speech_input == SP_MFCFILE || speech_input == SP_RAWFILE) {
-	file_counter++;
-      }
-
       VERMES("\n");
 
 #ifdef SP_BREAK_CURRENT_FRAME
@@ -526,7 +547,7 @@ main_recognition_loop()
       /* if this is the last segment, rest_param is NULL */
       if (rest_param != NULL) {
 	/* process the rest parameters in the next loop */
-	VERMES("<<<resuming>>>\n");
+	VERMES("<<<restart the rest>>>\n\n");
 	param = rest_param;
       } else {
 	/* input has reached end of stream loop, terminate program */
