@@ -7,7 +7,7 @@
 /* search_bestfirst_v1.c --- hypothesis handling and scoring functions
    for backscan algorithm */
 
-/* $Id: search_bestfirst_v1.c,v 1.9 2003/09/29 06:01:22 ri Exp $ */
+/* $Id: search_bestfirst_v1.c,v 1.11 2003/12/11 07:32:43 ri Exp $ */
 
 #include <julius.h>
 #ifndef PASS2_STRICT_IWCD
@@ -74,6 +74,9 @@ cpy_node(NODE *dst, NODE *src)
   dst->totallscore = src->totallscore;
 #endif
   dst->final_g = src->final_g;
+#ifdef VISUALIZE
+  dst->popnode = src->popnode;
+#endif
   return(dst);
 }
 
@@ -108,6 +111,10 @@ newnode()
     if (ccd_flag) tmp->g_prev[i] = LOG_ZERO;
   }
   tmp->final_g = LOG_ZERO;
+#ifdef VISUALIZE
+  tmp->popnode = NULL;
+#endif
+  tmp->tre = NULL;
 
   return(tmp);
 }
@@ -259,7 +266,7 @@ scan_word(NODE *now, HTK_Param *param)
   A_CELL *ac;
   WORD_ID word;
   LOGPROB tmpmax, tmptmp, score1;
-  int startt = 0;
+  int startt = 0, endt;
   int wordhmmnum;
   LOGPROB tmpmax_store, store_point_maxarc;
   int phmmlen;
@@ -620,6 +627,8 @@ scan_word(NODE *now, HTK_Param *param)
       if (tmpmax_store < LOG_ZERO) tmpmax_store = LOG_ZERO;
       now->g_prev[t] = tmpmax_store;
     }
+    /* store the number of last computed frame */
+    if (node_exist_p) endt = t;
 
     /* scan$B$7$?C18l$NBh#1%Q%9$G$N;OC<;~9o$h$j@h$^$G(B t $B$,?J$s$G$*$j!$$+$D(B
        $B$3$N(B t $B$K$*$$$F%9%3%"%(%s%Y%m!<%W$K$h$C$F@8$-;D$C$?%N!<%I$,0l$D$b(B
@@ -641,7 +650,7 @@ scan_word(NODE *now, HTK_Param *param)
     
   } /* end of time loop */
   
-  if (debug2_flag) j_printf("scanned: [%3d-%3d]\n",t+1,startt);
+  if (debug2_flag) j_printf("scanned: [%3d-%3d]\n", endt, startt);
 
  end_of_scan:
   /* $BA08~$-%9%3%"$N:G=*CM$r7W;;(B ($B>uBV(B 0 $B$+$i;~4V(B 0 $B$X$NA+0\(B) */
@@ -869,6 +878,7 @@ next_word(NODE *now, NODE *new,	NEXTWORD *nword, HTK_Param *param, BACKTRELLIS *
 	new->score = totalscore;
 	new->bestt = t;
 	new->estimated_next_t = tre->begintime - 1;
+	new->tre  =  tre;
       }
     }
   } else {
