@@ -7,7 +7,7 @@
 /* search_bestfirst_v2.c --- hypothesis handling and scoring functions
    for nextscan algorithm */
 
-/* $Id: search_bestfirst_v2.c,v 1.9 2003/09/29 06:01:22 ri Exp $ */
+/* $Id: search_bestfirst_v2.c,v 1.10 2003/12/11 07:32:43 ri Exp $ */
 
 /* by "fast" setting (default), search_bestfirst_v1.c is used for faster
    decoding.  Please specify option "--enable-setup=standard" or
@@ -73,6 +73,9 @@ cpy_node(NODE *dst, NODE *src)
   dst->totallscore = src->totallscore;
 #endif
   dst->final_g = src->final_g;
+#ifdef VISUALIZE
+  dst->popnode = src->popnode;
+#endif
   return(dst);
 }
 
@@ -104,6 +107,9 @@ newnode()
     tmp->g[i] = LOG_ZERO;
   }
   tmp->final_g = LOG_ZERO;
+#ifdef VISUALIZE
+  tmp->popnode = NULL;
+#endif
 
   return(tmp);
 }
@@ -626,6 +632,7 @@ next_word(NODE *now, NODE *new, NEXTWORD *nword, HTK_Param *param, BACKTRELLIS *
 	new->score = totalscore;
 	new->bestt = t;
 	new->estimated_next_t = tre->begintime - 1;
+	new->tre = tre;
       }
     }
   } else {
@@ -653,6 +660,7 @@ next_word(NODE *now, NODE *new, NEXTWORD *nword, HTK_Param *param, BACKTRELLIS *
       new->score = totalscore;
       new->bestt = t;
       new->estimated_next_t = tre->begintime - 1;
+      new->tre = tre;
     }
   }
   /* 2. search bckward */
@@ -671,6 +679,7 @@ next_word(NODE *now, NODE *new, NEXTWORD *nword, HTK_Param *param, BACKTRELLIS *
       new->score = totalscore;
       new->bestt = t;
       new->estimated_next_t = tre->begintime - 1;
+      new->tre = tre;
     }
   }
 
@@ -744,6 +753,7 @@ start_word(
 #endif
       new->score = new->g[peseqlen-1] + tre->backscore /*+ tmpp*/;
       new->estimated_next_t = tre->begintime - 1;
+      new->tre = tre;
       break;
     }
   }
