@@ -1,10 +1,12 @@
-/* Copyright (c) 1991-2002 Doshita Lab. Speech Group, Kyoto University */
-/* Copyright (c) 2000-2002 Speech and Acoustics Processing Lab., NAIST */
-/*   All rights reserved   */
+/*
+ * Copyright (c) 1991-2003 Kyoto University
+ * Copyright (c) 2000-2003 NAIST
+ * All rights reserved
+ */
 
 /* wchmm.c --- build lexicon tree (word-conjunction HMM) */
 
-/* $Id: wchmm.c,v 1.9 2002/09/11 22:02:33 ri Exp $ */
+/* $Id: wchmm.c,v 1.12 2003/09/29 06:01:22 ri Exp $ */
 
 /* wchmm = word conjunction HMM = lexicon tree */
 
@@ -715,11 +717,11 @@ get_nbest_uniprob(WORD_INFO *winfo, int n)
   /* store all unigram probability to u_p[] */
   u_p = (LOGPROB *)mymalloc(sizeof(LOGPROB) * winfo->num);
   for(w=0;w<winfo->num;w++) u_p[w] =
+			      uni_prob(ngram, winfo->wton[w])
 #ifdef CLASS_NGRAM
-			     class_uni_prob(ngram, winfo->wton[w]);
-#else
-                             uni_prob(ngram, winfo->wton[w]);
+			      + winfo->cprob[w]
 #endif
+			      ;
   /* sort them downward */
   qsort(u_p, winfo->num, sizeof(LOGPROB),
 	(int (*)(const void *,const void *))compare_prob);
@@ -833,10 +835,9 @@ build_wchmm(WCHMM_INFO *wchmm)
 #endif
 #ifdef SEPARATE_BY_UNIGRAM
     if (
-#ifdef CLASS_NGRAM
-       class_uni_prob(wchmm->ngram, wchmm->winfo->wton[i])
-#else
        uni_prob(wchmm->ngram, wchmm->winfo->wton[i])
+#ifdef CLASS_NGRAM
+       + wchmm->winfo->cprob[i]
 #endif
        >= separate_thres && separated_word_count < separate_wnum) {
       /* $BIQEY$N9b$$C18l$rLZ9=B$2=$7$J$$(B */
@@ -1045,10 +1046,9 @@ build_wchmm2(WCHMM_INFO *wchmm)
 #ifdef SEPARATE_BY_UNIGRAM
     /* separate high-frequent words from tree (threshold = separate_thres) */
     if (
-#ifdef CLASS_NGRAM
-	class_uni_prob(wchmm->ngram, wchmm->winfo->wton[i])
-#else
 	uni_prob(wchmm->ngram, wchmm->winfo->wton[i])
+#ifdef CLASS_NGRAM
+	+ wchmm->winfo->cprob[i]
 #endif
 	>= separate_thres && separated_word_count < separate_wnum) {
       wchmm_add_word(wchmm, i,0,0);
@@ -1175,7 +1175,7 @@ print_wchmm_info(WCHMM_INFO *wchmm)
 #ifdef USE_NGRAM
 #ifdef NO_SEPARATE_SHORT_WORD
 #ifdef SEPARATE_BY_UNIGRAM
-  j_printf(" (%d hi-freq. words are separated from tree)\n", separated_word_count);
+  j_printf("\t(%d hi-freq. words are separated from tree lexicon)\n", separated_word_count);
 #else
   j_printf(" (no words are separated from tree)\n");
 #endif /* SEPARATE_BY_UNIGRAM */
