@@ -6,7 +6,7 @@
 
 /* wchmm.h --- wchmm (word-conjunction HMM) lexicon tree */
 
-/* $Id: wchmm.h,v 1.6 2003/09/29 06:01:22 ri Exp $ */
+/* $Id: wchmm.h,v 1.7 2003/12/11 07:32:44 ri Exp $ */
 
 #ifndef __SENT_WORD_CONJ_HMM__
 #define __SENT_WORD_CONJ_HMM__
@@ -149,7 +149,7 @@ typedef struct __trellis_atom__ {
   WORD_ID wid;			/* word ID */
   short begintime;		/* beginning frame */
   short endtime;		/* end frame */
-#ifdef WORDGRAPH
+#ifdef WORD_GRAPH
   boolean within_wordgraph;	/* TRUE if within word graph */
 #endif
   struct __trellis_atom__ *last_tre; /* connection to previous trellis word */
