@@ -1,10 +1,12 @@
-/* Copyright (c) 1991-2002 Doshita Lab. Speech Group, Kyoto University */
-/* Copyright (c) 2000-2002 Speech and Acoustics Processing Lab., NAIST */
-/*   All rights reserved   */
+/*
+ * Copyright (c) 1991-2003 Kyoto University
+ * Copyright (c) 2000-2003 NAIST
+ * All rights reserved
+ */
 
 /* init_voca.c --- read in dict file */
 
-/* $Id: init_voca.c,v 1.3 2002/09/11 22:01:50 ri Exp $ */
+/* $Id: init_voca.c,v 1.5 2003/10/02 01:44:53 ri Exp $ */
 
 #include <sent/stddefs.h>
 #include <sent/htk_hmm.h>
@@ -16,7 +18,7 @@ init_voca(WORD_INFO *winfo, char *filename, HTK_HMM_INFO *hmminfo, boolean not_c
 {
   FILE *fd;
 
-  j_printerr("Reading in dictionary...");
+  j_printerr("Reading in dictionary...\n");
   if ((fd = fopen_readfile(filename)) == NULL) {
     j_printerr("failed to open %s\n",filename);
     return(FALSE);
@@ -25,7 +27,7 @@ init_voca(WORD_INFO *winfo, char *filename, HTK_HMM_INFO *hmminfo, boolean not_c
     if (force_dict) {
       j_printerr("errors are ignored\n");
     } else {
-      j_printerr("error in reading %s\n",filename);
+      j_printerr("error in reading %s: %d words failed out of %d words\n",filename, winfo->errnum, winfo->num);
       fclose_readfile(fd);
       return(FALSE);
     }
